from shipstation.api import *

from em_tasks.store.models import *
from em_tasks.store.product_source import ProductSource


class StoreUtil():
  stores = dict()

  def __init__(self):
    pass

  @classmethod
  def get_store_by_code(cls, store_code):
    if store_code not in cls.stores:
      cls.stores[store_code] = Store.get_or_none(Store.code == store_code)

    return cls.stores[store_code]

  @classmethod
  def save_store_product(cls, seller_id, source, source_product_id, product_attrs):
    s_product_id = product_attrs.pop('s_product_id', None)
    if not s_product_id:
      return

    store_product = StoreProduct.get_or_none(StoreProduct.s_product_id == s_product_id)
    if not store_product:
      store_product = StoreProduct(s_product_id=s_product_id)

    data = {
      'store_seller_id': seller_id,
      'source': source,
      'source_product_id': source_product_id,
    }
    data.update(product_attrs)
    for k, v in data.items():
      if v:
        setattr(store_product, k, v)
    store_product.save()

    for gp in GmcProduct.select().where(GmcProduct.handle == store_product.handle).execute():
      if gp.store_product_id != store_product.id:
        gp.store_product_id = store_product.id
        gp.save()

  @classmethod
  def delete_store_product(cls, product_id):
    StoreProduct.delete().where(StoreProduct.s_product_id == product_id).execute()

  @classmethod
  def get_amz_products(cls, marketplace):
    page = 1
    batch_size = 2500
    source = 'AMZ_{}'.format(marketplace.upper())
    query = StoreProduct.select().where(StoreProduct.source == source)
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      products = dict()
      for sp in query.paginate(page, batch_size).dicts():
        yield {'product_id': sp['s_product_id'], 'asin': sp['source_product_id'], 'handle': sp['handle']}

      page += 1

  @classmethod
  def convert_order_item_to_shipstation_order(self, order_item):
    order_key = None
    order_number = '{}-{}'.format(order_item.s_order_number, order_item.s_line_item_id)
    ss_order = ShipStationOrder(order_key, order_number)
    ss_order.set_order_date(order_item.s_purchase_date)

    status = 'awaiting_shipment'
    if order_item.canceled or order_item.refund_qty > 0:
      status = 'cancelled'
    elif order_item.s_tracking:
      status = 'shipped'

    ss_order.set_status(status)

    name = '{} {}'.format(order_item.first_name, order_item.last_name)
    ss_order.set_customer_details(name, order_item.email)

    shipping_address = ShipStationAddress(
      name, order_item.company, order_item.address1, order_item.address2, None,
      order_item.city, order_item.state, order_item.zipcode, order_item.country,
      order_item.phone)
    ss_order.set_shipping_address(shipping_address)

    ba = json.loads(order_item.billing_address)
    billing_address = ShipStationAddress(
      '{} {}'.format(ba['first_name'], ba['last_name']), ba['company'],
      ba['address1'], ba['address2'], None, ba['city'], ba['state'], ba['zipcode'],
      ba['country'], ba['phone'])
    ss_order.set_billing_address(billing_address)

    item = ShipStationItem(
      order_item.s_line_item_id, order_item.s_sku, order_item.s_name, order_item.s_image_url,
      order_item.s_quantity, order_item.s_price)
    ss_order.add_item(item)

    return ss_order

  @classmethod
  def convert_order_item_to_sheet_order(self, order_item):
    if not order_item.source:
      source_product_url = order_item.s_url
    elif order_item.source.startswith('EM'):
      source_product_url = order_item.s_url
    else:
      source_product_url = ProductSource.get_source_url(order_item.source, order_item.source_id)
    if order_item.refund_qty:
      if order_item.refund_qty == order_item.s_quantity:
        remark = 'Refunded'
      else:
        remark = 'Partial Refunded'
    else:
      if not order_item.s_product_id or not order_item.s_variant_id:
        remark = 'ProductDeleted'
      else:
        remark = ''
    return {
      'status': 'n',
      'order-id': '{}-{}'.format(order_item.s_order_id, order_item.s_line_item_id),
      'order-link': 'https://admin.shopify.com/store/d83075/orders/{}'.format(order_item.s_order_id),
      'recipient-name': '{} {}'.format(order_item.first_name, order_item.last_name),
      'purchase-date': order_item.s_purchase_date.isoformat(),
      'sku address': order_item.s_url,
      'sku': '{}-{}'.format(order_item.s_order_number, order_item.s_sku),
      'price': order_item.s_price,
      'quantity-purchased': order_item.s_quantity - order_item.refund_qty,
      'shipping-fee': order_item.s_shipping_charge,
      'ship-state': order_item.state,
      'isbn-address': source_product_url,
      'isbn': order_item.source_id,
      'SELLER': '',
      'SELLER-ID': '',
      'SELLER PRICE': '',
      'URL': '',
      'condition': 'new',
      'character': '',
      'Remark': '',
      'reference': '',
      'CODE': '',
      '0': '',
      'item-name': order_item.s_name,
      'ship-address-1': order_item.address1,
      'ship-address-2': order_item.address2 or '',
      'ship-city': order_item.city,
      'ship-zip': order_item.zipcode,
      'ship-phone-number': order_item.phone or '',
      'COST': '',
      'ORDER NUMBER': '',
      'ACCOUNT': '',
      'last code': '',
      'ship-country': order_item.country,
      'SID': '',
      'Sales Channel': '',
      'Condition': 'New-New',
      'Shipping Service': order_item.s_shipping_method or '',
      'Expected Ship Date': order_item.s_min_delivery_date or '',
      'Estimated Delivery Date': order_item.s_min_delivery_date or '',
      'Order Date': '',
      'Supplier Estimated Delivery Date': '',
      'Carrier': '',
      'Tracking Number': '',
      'Note1': '',
      'Note2': '',
      'Note3': '',
    }

  def save_product_seller(self, store_id, product_id, seller_id):
    pass

  def is_in_gmc(self, store_id, product):
    pass

  def upload_to_gmc(self, store_id, product):
    pass
