import datetime

from gspread.exceptions import APIError

from em_tasks import logger
from em_tasks.store.store_util import StoreUtil
from em_tasks.google.sheet_api import SheetAPI
from em_tasks.store.models import *


class ShopifyOrdersSheetExporter():
  sheet_cols = {
    'AMZ': [
      'status', 'order-id', 'recipient-name', 'purchase-date', 'sku address', 'sku', 'price',
      'quantity-purchased', 'shipping-fee', 'ship-state', 'isbn-address', 'isbn', 'SELLER',
      'SELLER-ID', 'SELLER PRICE', 'URL', 'condition', 'character', 'Remark', 'reference',
      'CODE', '0', 'item-name', 'ship-address-1', 'ship-address-2', 'ship-city', 'ship-zip',
      'ship-phone-number', 'COST', 'ORDER NUMBER', 'ACCOUNT', 'last code', 'ship-country',
      'SID', 'Sales Channel', 'Condition', 'Shipping Service', 'Expected Ship Date',
      'Estimated Delivery Date', 'Order Date', 'Supplier Estimated Delivery Date', 'Carrier',
      'Tracking Number', 'Note1', 'Note2', 'Note3'
    ],
    'Ebay': [
      'order-link', 'order-id', 'recipient-name', 'ship-address-1', 'ship-address-2', 'ship-city',
      'ship-state', 'ship-zip', 'ship-country', 'ship-phone-number', 'purchase-date', 'sku',
      'item-name', 'options', 'sku address', 'isbn-address', 'price', 'quantity-purchased',
      'ORDER COST', 'ORDER NUMBER', 'ACCOUNT', 'last code', 'Order Date', 'Remark', 'SELLER',
      'SELLER PRICE', 'Carrier', 'Tracking Number', 'Supplier Estimated Delivery Date', 'New link',
      'Ship to Customer', 'Gift', 'Operater'
    ],
    'Rakuten': [
      'status', 'order-id', 'recipient-name', 'purchase-date', 'sku address', 'sku', 'price',
      'quantity-purchased', 'shipping-fee', 'ship-state', 'isbn-address', 'isbn', 'SELLER',
      'SELLER-ID', 'SELLER PRICE', 'URL', 'condition', 'character', 'Remark', 'reference',
      'CODE', '0', 'item-name', 'ship-address-1', 'ship-address-2', 'ship-city', 'ship-zip',
      'ship-phone-number', 'COST', 'ORDER NUMBER', 'ACCOUNT', 'last code', 'ship-country',
      'SID', 'Sales Channel', 'Condition', 'Shipping Service', 'Expected Ship Date',
      'Estimated Delivery Date', 'Order Date', 'Supplier Estimated Delivery Date', 'Carrier',
      'Tracking Number', 'Note1', 'Note2', 'Note3'
    ],
    'Coupang': [
      'status', 'order-id', 'purchase-date', 'sku', 'sku address', 'price', 'quantity-purchased',
      'isbn-address', 'item-name', 'options', 'COST', 'Carrier', 'Tracking Number', 'Order Date',
      'ORDER NUMBER', 'recipient-name', 'ship-address-1', 'ship-address-2', 'ship-city',
      'ship-state', 'ship-country', 'ship-zip', 'ship-phone-number', 'Note1', 'Note2', 'Note3'
    ],
    'EMP': [
      'order-link', 'order-id', 'recipient-name', 'ship-address-1', 'ship-address-2', 'ship-city',
      'ship-state', 'ship-zip', 'ship-country', 'ship-phone-number', 'purchase-date', 'item-name',
      'sku address', 'sku', 'price', 'quantity-purchased', 'shipping-fee', 'isbn-address',
      'ORDER COST', 'ORDER NUMBER', 'ACCOUNT', 'Order Date', 'SELLER', 'SELLER PRICE', 'Carrier',
      'Tracking Number', 'Remark'
    ],
  }

  spreadsheet_ids = {
    1: {
      'EMP': '1ZtOnb1fnmCT8pEg58WXmV0QlTb1VJYA8JVVzTJgyTFc',
      'Ebay': '1-tSAKaS8KeIWf_TZcGEcc7T49JU9jWXm2HL2O3HTthw',
      'Ebay_US': '1-tSAKaS8KeIWf_TZcGEcc7T49JU9jWXm2HL2O3HTthw',
      'AMZ_US': '1r_h4iU1iRovGG2XB9VIiQejtSY7O8nMdfro9jAvX1sU',
      'Coupang': '1gIC4wB4U8Nwz945Oi6ed53zbtANH1hiIYzbXt-2U5NM'
    },
    3: {
      'AMZ_US': '1zw7xzh1co7Gbgz1-eaoRCphp_bvZjb6PtqxaKSU96UI'
    },
    4: {
      'AMZ_JP': '1TrUL9MUwjHaR-h4yIG6VCbIlaK87FZP08lijhxVELeM',
      'Rakuten': '1TrUL9MUwjHaR-h4yIG6VCbIlaK87FZP08lijhxVELeM'
    },
    5: {
      'Coupang': '1gIC4wB4U8Nwz945Oi6ed53zbtANH1hiIYzbXt-2U5NM'
    },
    6: {
      'Ebay': '1-tSAKaS8KeIWf_TZcGEcc7T49JU9jWXm2HL2O3HTthw',
      'Ebay_US': '1JyfmUPR4BIQXYwpPtNjFkAr9ac4kajak82CJHiq54dE'
    },
    7: {
      'AMZ_JP': '1sgwjRuw9ASeY6KDS94Xc4ke346mzN-welNHZS_au0bE',
      'Rakuten': '1sgwjRuw9ASeY6KDS94Xc4ke346mzN-welNHZS_au0bE'
    },
    8: {
      'AMZ_UK': '1JyfmUPR4BIQXYwpPtNjFkAr9ac4kajak82CJHiq54dE'
    }
  }

  def __init__(self, service_account_data):
    self.service_account_data = service_account_data
    self.sheet_api = SheetAPI(service_account_data)

  def run(self):
    sheet_name = (datetime.datetime.utcnow() - datetime.timedelta(hours=5)).strftime('%m/%d')
    spreadsheets = dict()
    orders = OrderItem.select().where((OrderItem.risky == 0) & (OrderItem.exported == 0)).execute()
    for order in orders:
      sheet_order = StoreUtil.convert_order_item_to_sheet_order(order)
      spreadsheet_id = None
      if not order.vendor_id or not order.source:
        spreadsheet_id = self.spreadsheet_ids[1]['EMP']
      else:
        if order.vendor_id not in self.spreadsheet_ids:
          logger.debug('[OrderNoSpreadsheet] %s', sheet_order)
          continue

        if order.source not in self.spreadsheet_ids[order.vendor_id]:
          logger.debug('[OrderNoSpreadsheet] %s', sheet_order)
          continue

        spreadsheet_id = self.spreadsheet_ids[order.vendor_id][order.source]

      if not spreadsheet_id:
        continue

      if spreadsheet_id not in spreadsheets:
        spreadsheets[spreadsheet_id] = {'spreadsheet': self.sheet_api.get_spreadsheet(spreadsheet_id)}

      spreadsheet = spreadsheets[spreadsheet_id]['spreadsheet']

      spreadsheet_data = spreadsheets[spreadsheet_id]
      if sheet_name not in spreadsheet_data:
        spreadsheet_data[sheet_name] = self.sheet_api.find_or_create_sheet(spreadsheet, sheet_name)

      worksheet = spreadsheet_data[sheet_name]

      cols = None
      if not order.source:
        cols = self.sheet_cols['AMZ']
      elif order.source in self.sheet_cols:
        cols = self.sheet_cols[order.source]
      elif order.source.startswith('AMZ'):
        cols = self.sheet_cols['AMZ']
      elif order.source.startswith('Ebay'):
        cols = self.sheet_cols['Ebay']
      elif order.source.startswith('Rakuten'):
        cols = self.sheet_cols['Rakuten']
      elif order.source.startswith('EM'):
        cols = self.sheet_cols['EMP']
      else:
        cols = self.sheet_cols['AMZ']

      record = dict()
      for idx, col in enumerate(cols):
        record[idx] = sheet_order.get(col, '')

      key = sheet_order['order-id']
      while True:
        try:
          self.sheet_api.append_row_by_key(worksheet, key, record)
          logger.info(
            '[OrderExported] OrderID: %s, Order: %s, SheetID: %s, SheetName: %s',
            key, sheet_order['sku'], spreadsheet_id, sheet_name)

          order.exported = True
          order.sheet_id = spreadsheet_id
          order.sheet_name = sheet_name
          order.save()

          break
        except APIError as e:
          logger.exception(e)
          logger.info('[WriteSheetThrottle] Wait 60s to retry')
          time.sleep(60)
