# -*- coding: utf-8 -*-

from em_tasks.store.store_util import StoreProduct


class ShopifyProductsIndexer():
  def __init__(self, product_service):
    self.product_service = product_service
    self.indice_name = 'shopify_products'

  def run(self):
    page = 1
    batch_size = 2500
    query = StoreProduct.select().order_by(StoreProduct.id.asc())
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      products = []
      for sp in query.paginate(page, batch_size).dicts():
        products.append(sp)

      self.process_products(products)

      page += 1

  def process_products(self, products):
    self.product_service.ensure_indice(self.indice_name)

    buf_size = 500
    prod_buf = []
    for product in products:
      product.pop('id', None)
      product.pop('store_seller_id', None)
      product['product_id'] = '{}-{}'.format(product['source'], product['source_product_id'])
      prod_buf.append(product)

      if len(prod_buf) < buf_size:
        continue

      self.product_service.save_products(self.indice_name, prod_buf)
      prod_buf = []

    if prod_buf:
      self.product_service.save_products(self.indice_name, prod_buf)
      prod_buf = []
