import datetime
import re

import dateutil
import dateutil.parser

from em_tasks import logger
from em_tasks.google.sheet_api import SheetAPI


class OrderReader(object):
  pass


class GSheetOrderReader(OrderReader):
  def __init__(self, sheet_api):
    self.sheet_api = sheet_api

  def read(self, spreadsheet_id, days=7, include_today=True):
    today = datetime.datetime.utcnow()
    if include_today:
      to_date = today
    else:
      to_date = today - datetime.timedelta(days=1)
    from_date = today - datetime.timedelta(days=days)

    logger.debug('[SheetProcessing] %s', spreadsheet_id)

    spreadsheet = self.sheet_api.get_spreadsheet(spreadsheet_id)
    if not spreadsheet:
      logger.error('[SpreadSheetNotFound] %s', spreadsheet_id)
      return

    worksheets = self.sheet_api.get_worksheets(spreadsheet)
    for worksheet in worksheets:
      records = []
      name = worksheet.title
      try:
        sheet_date = dateutil.parser.parse(name)

        if not sheet_date or sheet_date < from_date or sheet_date > to_date:
          # logger.debug('[SheetIgnored] %s', name)
          continue
      except Exception as e:
        # None date sheets
        continue

      logger.debug('[Sheet] SheetID: %s, SheetName: %s', spreadsheet_id, name)

      vals = self.sheet_api.get_worksheet_vals(worksheet)
      headers = None
      for row in vals:
        if not headers:
          if 'tracking' not in row and 'Tracking' not in row and 'Tracking Number' not in row:
            continue

          headers = [col.lower() for col in row]
          continue

        empty = True
        for col in row:
          if col:
            empty = False
            break

        if empty:
          continue

        record = {'spreadsheet_id': spreadsheet_id, 'sheet': name}
        for idx, header in enumerate(headers):
          try:
            record[header] = row[idx]
          except Exception as e:
            logger.exception(e)

        order_number = None
        item_sku = None
        if 'sku' in record and record['sku']:
          if record['sku'].startswith('EM'):
            order_number = '-'.join(record['sku'].split('-')[:2])
            item_sku = record['sku'].replace('{}-'.format(order_number), '')
        elif 'sales channel' in record and record['sales channel']:
          order_number = '-'.join(record['sales channel'].split('-')[0:2])
        elif 'em order number' in record and record['em order number']:
          order_number = record['em order number']
        else:
          logger.info('[OrderNumberNotFound] %s', record)
          continue

        if not order_number:
          continue

        record['order_number'] = order_number
        record['item_sku'] = item_sku if item_sku else record.get('sku', '')
        records.append(record)

      yield (worksheet, headers, records)
