import json

from em_tasks.shopify.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil


class ShopifyAmzRepricer():
  def __init__(self, store_code, marketplace, offer_service, price_caculator):
    self.condition = 'new'

    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['name'], credential['token'], credential.get('release', None))

    self.marketplace = marketplace
    self.offer_service = offer_service
    self.price_calc = price_caculator

    # self.gss = dict()
    # gmc_stores = GmcStore.select().execute()
    # for gmc_store in gmc_stores:
    #   self.gss[gmc_store.merchant_id] = GoogleShopping(
    #     merchant_id, json.loads(gmc_store.service_account_data))

  def run(self):
    batch_size = 250
    prods = dict()
    for amz_prod in StoreUtil.get_amz_products(self.marketplace):
      prods[amz_prod['product_id']] = amz_prod
      if len(prods) < batch_size:
        continue

      self.process_products(prods)
      prods.clear()

    if prods:
      self.process_products(prods)
      prods.clear()

  def process_products(self, prods):
    offers = dict()
    asins = [prod['asin'] for prod in prods.values()]
    src_offers = self.offer_service.get_offers(self.marketplace, asins, self.condition)
    if not src_offers:
      return

    for asin, src_offer in src_offers.items():
      offers[asin] = {'asin': asin, 'src_offer': src_offer}
      offer = self.price_calc.calc_offer(src_offer)
      if offer:
        offers[asin].update(offer)
      else:
        offers[asin] = offer

    for _, prod in prods.items():
      asin = prod['asin']
      if asin in offers:
        offer = offers[asin]
        prod['offer'] = offer

    self.product_util.set_products_offer(prods)
    # store_offers = self.product_util.set_products_offer(prods)

    # items = dict()
    # for prod in prods.values():
    #   items[prod['handle']] = prod

    # handles = list(items.keys())
    # gmc_offers = dict()
    # for gmc_product in GmcProduct.select().where(GmcProduct.handle.in_(handles)).execute():
    #   if gmc_product.handle not in items:
    #     continue

    #   gmc_price = round(gmc_product.price, 2)
    #   gmc_availability = gmc_product.availability
    #   if gmc_availability != gmc_offers['availability'] or gmc_price != gmc_offers['price']:
    #     pass