# -*- coding: utf-8 -*-

import json

from em_tasks.shopify.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil
from em_tasks.utils.product_converters import AmzProductConverter


class ShopifyAmzUpdater():
  def __init__(self, store_code, marketplace, product_service):
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['name'], credential['token'], credential.get('release', None))

    self.marketplace = marketplace.lower()
    self.product_service = product_service
    self.product_converter = AmzProductConverter()
    self.indice_name = 'amz_products_api_{}'.format(marketplace.lower())

  def run(self):
    batch_size = 250
    prods = dict()
    for amz_prod in StoreUtil.get_amz_products(self.marketplace):
      prods[amz_prod['product_id']] = amz_prod
      if len(prods) < batch_size:
        continue

      self.process_products(prods)
      prods.clear()

    if prods:
      self.process_products(prods)
      prods.clear()

  def process_products(self, prods):
    asins = [prod['asin'] for prod in prods.values()]
    products_info = self.product_service.search_products(self.indice_name, asins)
    if not products_info:
      return

    records = dict()
    for asin, product_info in products_info.items():
      data = self.product_converter.convert(product_info)
      if self.marketplace != 'us':
        data.pop('title', None)

      records[asin] = data

    for _, prod in prods.items():
      asin = prod['asin']
      if asin in records:
        prod['data'] = records[asin]

    self.product_util.update_products(prods)
